@@ -25,13 +25,13 @@ sub report (@x) returns Str is export {
 
 #Autrijus' "cls"
 sub clear returns Void is export{
-    system(($?OS eq any<MSWin32 mingw>) ?? 'cls' :: 'clear');
+    system(($?OS eq any<MSWin32 mingw>) ?? 'cls' !! 'clear');
 }
 
 sub max (Array @x) returns Int is export{
     my $max;
     for @x->$try{
-        $max =    $max > $try ?? $max :: $try; 
+        $max =    $max > $try ?? $max !! $try; 
     }
     $max.int
 }    
