@@ -88,10 +88,10 @@ sub header (
     }
     
     if ($cookies) {
-        my @cookies = ($cookies !~ Array) ?? ($cookies) :: @{$cookies};
+        my @cookies = ($cookies !~ Array) ?? ($cookies) !! @{$cookies};
         
         for @cookies -> $cookie {
-            #$cookie = ($cookie ~~ CGI::Cookie) ?? $cookie.as_string :: $cookie;
+            #$cookie = ($cookie ~~ CGI::Cookie) ?? $cookie.as_string !! $cookie;
             
             $header ~= "\nSet-Cookie: " ~ $cookie
                 unless $cookie eq "";
