@@ -71,10 +71,10 @@ submethod BUILD ($self: Str *%headers) {
 }
 
 method header ($self: Str $field) is rw {
-    return Proxy.new(
-	FETCH => { %:headers{$field} },
-	STORE => -> Str $val { $self.:header($field, $val) }
-    );
+  return Proxy.new(
+    FETCH => { %:headers{$field} },
+    STORE => -> Str $val { $self.:header($field, $val) }
+  );
 }
 
 method clear () { %:headers = () }
@@ -93,7 +93,7 @@ method remove_header (Str *@fields) {
   for @fields -> $field is copy {
     $field ~~ tr/_/-/ if not $field ~~ /^\:/ and $TRANSLATE_UNDERSCORE;
     my $v = %:headers.delete($field.lc);
-    push @values, $v ~~ Array ?? @$v :: $v if defined $v;
+    push @values, $v ~~ Array ?? @$v !! $v if defined $v;
   }
 
   return @values;
@@ -124,17 +124,17 @@ method :header (Str $field is copy, Str $val is copy, Str ?$op = "") {
 
   my $h = %:headers{$field};
   $h //= [];
-  my @old = $h ~~ Array ?? @$h :: ($h);
+  my @old = $h ~~ Array ?? @$h !! ($h);
 
   $val = undef if $op eq "INIT" and @old;
   if $val.defined {
-    my @new = ($op eq "PUSH") ?? @old :: ();
+    my @new = ($op eq "PUSH") ?? @old !! ();
     if $val !~ Array {
       push @new, $val;
     } else {
       push @new, *@$val;
     }
-    %:headers{$field} = @new > 1 ?? \@new :: @new[0];
+    %:headers{$field} = @new > 1 ?? \@new !! @new[0];
   }
 
   return @old;
@@ -158,7 +158,7 @@ method scan ($self: Code $sub) {
     my $vals = %:headers{$key};
     if $vals ~~ Array {
       for @$vals -> $val {
-	$sub.(%standard_case{$key} || $key, $val);
+        $sub.(%standard_case{$key} || $key, $val);
       }
     } else {
       $sub.(%standard_case{$key} || $key, $vals);
@@ -222,13 +222,13 @@ method content_type ($self: ) is rw {
 
       my @ct = split /;\s*/, $ct, 2;
       given @ct[0] {
-	s:g/\s+//;
-	$_ .= lc;
+        s:g/\s+//;
+        $_ .= lc;
       }
 
       given want {
-	when List   { return @ct }
-	when Scalar { return @ct[0] }
+        when List { return @ct }
+        when Item { return @ct[0] }
       }
     },
     STORE => -> Str $type {
@@ -241,13 +241,13 @@ method referer ($self: ) is rw {
     FETCH => { @{$self.:header("Referer")}[0] },
     STORE => -> $new is copy {
       if ($new ~~ /\#/) {
-	# Strip fragment per RFC 2616, section 14.36.
-	if ($new ~~ URI) {
-	  $new .= clone;
-	  $uri.fragment = undef;
-	} else {
-	  $new ~~ s/\#.*//;
-	}
+        # Strip fragment per RFC 2616, section 14.36.
+        if ($new ~~ URI) {
+          $new .= clone;
+          $uri.fragment = undef;
+        } else {
+          $new ~~ s/\#.*//;
+        }
       }
 
       $self.:header("Referer", $new);
@@ -282,12 +282,12 @@ method :basic_auth ($self: Str $h) is rw {
     FETCH => {
       my $cur = $self.:header($h);
       if (defined $old and $old ~~ s/^\s* Basic \s+//) {
-	#my $val = MIME::Base64::decode($cur);
+        #my $val = MIME::Base64::decode($cur);
 
-	given want {
-	  when Scalar { return $val }
-	  when List   { return split /\:/, $val, 2 }
-	}
+        given want {
+          when Item { return $val }
+          when List { return split /\:/, $val, 2 }
+        }
       }
 
       return undef;
