@@ -2,7 +2,7 @@
 use v6;
 
 use HTTP::Status ();
-require HTTP::Message;
+use HTTP::Message;
 
 class HTTP::Response[?::URI_CLASS = URI] {
     is HTTP::Message;
