@@ -5,10 +5,15 @@ module Pugs.AST.Pos (
 ) where
 import Data.Typeable
 
--- |Represents the position of a chunk of source code: filename; start
--- line & column; end line & column.
+{-|
+Represents the position of a chunk of source code: filename; start
+line & column; end line & column.
+
+Not to be confused with the 'Exp' constructor of the same name, which stores
+a sub-tree and its associated 'Pos'.
+-}
 data Pos = MkPos
-    { posName           :: !String
+    { posName           :: !String -- ^ Source file name
     , posBeginLine      :: !Int
     , posBeginColumn    :: !Int
     , posEndLine        :: !Int
