@@ -37,7 +37,7 @@ sub exit (Int ?$status = 0) is builtin is primitive {
 sub Perl6::Internals::eval_parrot (Str $code) is builtin is primitive {
     my $sub = substr($code, 0, 1) eq "."
         ?? Perl6::Internals::compile_pir($code)
-        :: Perl6::Internals::compile_pir(".sub pugs_eval_parrot\n$code\n.end\n");
+        !! Perl6::Internals::compile_pir(".sub pugs_eval_parrot\n$code\n.end\n");
     $sub();
 }
 
@@ -59,7 +59,7 @@ sub shift (@a) is builtin is primitive {
     my $e = @a[0];
     my $i = 0;
     while $i < $top {
-	@a[$i++] = @a[$i];
+        @a[$i++] = @a[$i];
     }
     pop(@a);
     return $e;
@@ -72,8 +72,8 @@ sub splice (@a is rw, ?$offset=0, ?$length, *@list) is builtin is primitive {
 
     $off += $size if $off < 0;
     if $off > $size {
-	warn "splice() offset past end of array\n";
-	$off = $size;
+        warn "splice() offset past end of array\n";
+        $off = $size;
     }
     # $off is now ready
 
@@ -88,46 +88,46 @@ sub splice (@a is rw, ?$offset=0, ?$length, *@list) is builtin is primitive {
     my @result;
 
     if 1 {
-	my $i = $off;
-	my $stop = $off + $len;
-	while $i < $stop {
-	    push(@result,@a[$i]);
-	    $i++;
-	}
+        my $i = $off;
+        my $stop = $off + $len;
+        while $i < $stop {
+            push(@result,@a[$i]);
+            $i++;
+        }
     }
 
     if $size_change > 0 {
-	my $i = $size + $size_change -1;
-	my $final = $off + $size_change;
-	while $i >= $final {
-	    @a[$i] = @a[$i-$size_change];
-	    $i--;
-	}
+        my $i = $size + $size_change -1;
+        my $final = $off + $size_change;
+        while $i >= $final {
+            @a[$i] = @a[$i-$size_change];
+            $i--;
+        }
     } elsif $size_change < 0 {
-	my $i = $off;
-	my $final = $size + $size_change -1;
-	while $i <= $final {
-	    @a[$i] = @a[$i-$size_change];
-	    $i++;
-	}
-	# +@a = $size + $size_change;
-	#   doesnt exist yet, so...
-	my $n = 0;
-	while $n-- > $size_change {
-	    pop(@a);
-	}
+        my $i = $off;
+        my $final = $size + $size_change -1;
+        while $i <= $final {
+            @a[$i] = @a[$i-$size_change];
+            $i++;
+        }
+        # +@a = $size + $size_change;
+        #   doesnt exist yet, so...
+        my $n = 0;
+        while $n-- > $size_change {
+            pop(@a);
+        }
     }
 
     if $listlen > 0 {
-	my $i = 0;
-	while $i < $listlen {
-	    @a[$off+$i] = @list[$i];
-	    $i++;
-	}
+        my $i = 0;
+        while $i < $listlen {
+            @a[$off+$i] = @list[$i];
+            $i++;
+        }
     }
 
-    #  want.List ?? *@result :: pop(@result)
-    #  want.List ?? *@result :: +@result ?? @result[-1] :: undef;
+    #  want.List ?? *@result !! pop(@result)
+    #  want.List ?? *@result !! +@result ?? @result[-1] !! undef;
     #  *@result;
     @result;
 }
