@@ -10,9 +10,11 @@ use Config ();
 # Sets up either the Null Prelude placeholder, or a real precompiled
 # AST of Prelude.pm.
 
-our $TEMP_PRELUDE = "tmp-Prelude.pm"; # XXX: move this to config.yml?
+our %Config;
+our $TEMP_PRELUDE = "Prelude.pm"; # XXX: move this to config.yml?
+END { unlink $TEMP_PRELUDE unless $Config{keep} };
 
-GetOptions \our %Config, qw(--null --pugs|p=s --inline|i=s@ --verbose|v --touch --output|o=s);
+GetOptions \%Config, qw(--null --pugs|p=s --inline|i=s@ --verbose|v --touch --output|o=s --keep|k);
 setup_output();
 
 touch() if $Config{touch};
@@ -24,10 +26,10 @@ exit 1;
 
 sub setup_output {
     if ($Config{output}) {
-        open $Config{out_fh}, ">", $Config{output} or
+        open OUT, "> $Config{output}" or
             die "open: $Config{output}: $!";
     } else {
-        $Config{out_fh} = \*STDOUT;
+        *OUT = *STDOUT;
     }
 }
 
@@ -39,15 +41,19 @@ sub touch {
     print STDERR "Triggering rebuild... " if $Config{verbose};
     unlink "src/Pugs/Run.hi";
     unlink "src/Pugs/Run.o";
+    unlink "dist/build/Pugs/Run.hi";
+    unlink "dist/build/Pugs/Run.o";
+    unlink "dist/build/src/Pugs/Run.hi";
+    unlink "dist/build/src/Pugs/Run.o";
     #unlink "pugs$Config::Config{_exe}";
     print STDERR "done.\n" if $Config{verbose};
 }
 
 sub null {
     print STDERR "Generating null Prelude... " if $Config{verbose};
-    open my $np, "src/Pugs/PreludePC.hs-null" or
+    open NP, "src/Pugs/PreludePC.hs-null" or
         die "Couldn't open null Prelude (src/Pugs/PreludePC.hs-null): $!";
-    print { $Config{out_fh} } $_ while (<$np>);
+    print OUT <NP>;
     print STDERR "done.\n" if $Config{verbose};
 }
 
@@ -101,7 +107,7 @@ sub precomp {
 
     exit 1 unless length $program;
 
-    print { $Config{out_fh} } <<'.';
+    print OUT <<'.';
 {-# OPTIONS_GHC -fglasgow-exts -fno-full-laziness -fno-cse #-}
 {-
     *** NOTE ***
@@ -154,10 +160,10 @@ initPreludePC env = do
 
     $program =~ s/.*^globC/globPCP :: IO Pad\nglobPCP/ms;
     $program =~ s/^expC/astPCP :: IO Exp\nastPCP/ms;
-    print { $Config{out_fh} } $program;
+    print OUT $program;
 
     die "Pugs ".(($?&255)?"killed by signal $?"
-		 :"exited with error code ".($?>>8)) if $?;
+         :"exited with error code ".($?>>8)) if $?;
     print STDERR "done.\n" if $Config{verbose};
 }
 
